/*
 * SM630.c
 *
 *  Created on: Sep 11, 2014
 *      Author: Zeus @ Sisoog.com
 */

#include <stdint.h>
#include "Uart.h"
#include "SM630.h"
#include <string.h>


#define SM_SEND(x)		Uart3_Send(x)
#define SM_Get()		Uart3_Get();



int OS_GetTime(void)
{
	extern volatile uint32_t SYSTime;
	return SYSTime;
}

uint8_t finger_Get_pack(uint8_t *buffer,uint8_t Max_Len,uint32_t TimeOut)
{
	uint32_t Start_Time = OS_GetTime();
	uint8_t phase = 0;
	uint8_t Write_index = 0;


	while((OS_GetTime()-Start_Time)<TimeOut)
	{
		char Data;
		if(Uart3_Get_char(&Data))
		{
			switch(phase+2)
			{
			case 0:
				if(Data==0x4D)
					phase = 1;
				break;
			case 1:
				if(Data==0x58)
				{
					phase = 2;
					buffer[0] = 0x45;
					buffer[1] = 0x58;
				}
				else
					phase = 0;
				break;
			case 2:
				buffer[2+Write_index] = Data;
				if(Write_index>1 && buffer[8+1]<=(Write_index-0xA))
				{
					return 0;
				}
				Write_index++;
			}
		}
	}
	return 1;
}

uint8_t finger_send_Command(SM_Command cmd,uint8_t *Data,uint8_t Data_len)
{
    int i;
    uint8_t Buf[20] ={0};
	uint8_t	sum = 0xB6;
	Data_len++;

	SM_SEND(0x4D);
	SM_SEND(0x58);
	SM_SEND(0x10); /*For Packet*/
	SM_SEND(Data_len);
	sum+=Data_len;
	SM_SEND(cmd);
	sum+=cmd;
	for(i=0;i<Data_len-8;i++)
	{
		SM_SEND(Data[i]);
		sum+=Data[i];
	}
	SM_SEND(sum);


	
	if(finger_Get_pack(Buf,15,200)==0 && Buf[4]==0x01)
		return 0;
	return 1;
}


uint8_t finger_Read_info(uint8_t *Buffer)
{
	uint8_t Buf[32]={0};
	if(finger_send_Command(read_product_logo,Buf,0)==0)
	{
		if(finger_Get_pack(Buf,31,200)==0)
		{
			memset(Buffer,0,25);
			memcpy(Buffer,&Buf[5],24);
			return 0;
		}
	}
	return 1;
}


uint8_t finger_Read_IDnum(uint8_t *Buffer)
{
	uint8_t Buf[32]={0};
	if(finger_send_Command(read_id_number,Buf,0)==0)
	{
		if(finger_Get_pack(Buf,31,200)==0)
		{
			memset(Buffer,0,250);
			memcpy(Buffer,&Buf[5],204);
			return 0;
		}
	}
	return 1;
}


uint8_t finger_Write_flash(uint16_t Adders,uint8_t *Data,uint8_t data_Len)
{
	uint8_t AH = Adders/8;
	uint8_t AL = Adders%8;
	uint8_t Buffer[138];
	Buffer[0] = AH;
	Buffer[1] = AL;
	Buffer[2] = data_Len;
	memcpy(&Buffer[3],Data,data_Len);
	if(finger_send_Command(write_user_flash,Buffer,data_Len+3)==0)
	{
		memset(Buffer,0,50);
		if(finger_Get_pack(Buffer,31,200)==0 && Buffer[2]==0x30 && Buffer[5]==0x31)
		{
			return 0; /*Ok */
		}
	}

	return 1;
}

uint8_t finger_Read_flash(uint16_t Adders,uint8_t *Data,uint8_t data_Len)
{
	uint8_t AH = Adders/8;
	uint8_t AL = Adders%8;
	uint8_t Buffer[130];
	Buffer[0] = AH;
	Buffer[1] = AL;
	Buffer[2] = data_Len;

	if(finger_send_Command(read_user_flash,Buffer,3)==0)
	{
		memset(Buffer,0,130);
		if(finger_Get_pack(Buffer,130,200)==0 && Buffer[2]==0x30 && Buffer[4]==read_user_flash)
		{
			memcpy(Data,&Buffer[5],data_Len);
			return 0; /*Ok */
		}
	}
	return 1;
}


uint8_t finger_delete(uint16_t ID)
{
	uint8_t AH = ID/8;
	uint8_t AL = ID%8;
	uint8_t Buffer[15];
	Buffer[0] = AH;
	Buffer[1] = AL;

	if(finger_send_Command(delet_fingerprint,Buffer,2)==0)
	{
		memset(Buffer,0,15);
		if(finger_Get_pack(Buffer,15,500)==0)
		{
			if(Buffer[2]==0x30 && Buffer[4]==delet_fingerprint && Buffer[5]==0x31)
				return 0; /*OK*/

		}
	}
	return 1;
}

uint8_t finger_add(uint16_t ID)
{
	uint8_t AH = ID/8;
	uint8_t AL = ID%8;
	uint8_t Buffer[15];
	Buffer[0] = AH;
	Buffer[1] = AL;

	if(finger_send_Command(add_fingerprint,Buffer,2)==0)
	{
		memset(Buffer,0,15);
		if(finger_Get_pack(Buffer,15,10000)==0)
		{
			if(Buffer[2]==0x30 && Buffer[4]==add_fingerprint)
			{
				if(Buffer[5]==0x31)
				{
					memset(Buffer,0,15);
					if(finger_Get_pack(Buffer,15,5000)==0)
					{
						if(Buffer[2]==0x30 && Buffer[4]==add_fingerprint && Buffer[5]==0x31)
							return 0; /*OK*/
					}

				}
				else if(Buffer[5]==0x33)
				{
					return 1; /*It's Timeout*/
				}

			}

		}
	}
	return 1;
}


uint8_t finger_search(uint16_t Start_ID,uint16_t NumID,uint16_t *MatchID)
{

	uint8_t Buffer[15];
	Buffer[0] = Start_ID/8;
	Buffer[1] = Start_ID%8;
	Buffer[2] = NumID/8;
	Buffer[3] = NumID%8;

	if(finger_send_Command(search_fingerprint,Buffer,4)==0)
	{
		memset(Buffer,0,15);
		if(finger_Get_pack(Buffer,15,10500)==0)
		{
			if(Buffer[2]==0x30 && Buffer[4]==search_fingerprint)
			{
				if(Buffer[5]==0x31) /*Found ID*/
				{
					memset(Buffer,0,15);
					if(finger_Get_pack(Buffer,15,2000)==0)
					{
						if(Buffer[5]==0x39)
						{
							*MatchID = Buffer[6]*256+Buffer[7];
							return 0;
						}
						else if(Buffer[5]==0x3A)
						{
							return Not_Found_Match_ID;
						}
						else if(Buffer[5]==0x34)
						{
							return finger_print_is_poor;
						}

					}
				}
				else if(Buffer[5]==0x33)
				{
					return Req_TimeOut; /*It's Timeout*/
				}

			}
		}
	}

	return 1;
}



