/*
 * Lcd_Driver.c
 *
 *  Created on: Sep 9, 2014
 *      Author: Zeus @ Sisoog.com
 */
#include "Latch_595_Driver.c"
#include <delay.h>




uint8_t LCD_Pin_Data=0;

#define delay_us(x)		delay_us(x)
#define delay_ms(x)		delay_ms(x)



#define LCD_LINE0_DDRAMADDR		0x00
#define LCD_LINE1_DDRAMADDR		0x40
#define LCD_LINE2_DDRAMADDR		0x14
#define LCD_LINE3_DDRAMADDR		0x54
#define LCD_DDRAM           7      // DB7: set DD RAM address
#define LCD_CGRAM           6      // DB6: set CG RAM address
#define LCD_CLR             0      // DB0: clear display
#define LCD_HOME            1      // DB1: return to home position

/*
void delay_us(int x)
{
	volatile int i;
	for(i=0;i<1000*x;i++);
}
*/


void SetLCD_RW(uint8_t stat)
{

}


void SetLCD_RS(uint8_t stat)
{
	if(stat==0)
	{
		LCD_Pin_Data &= ~LCD_RS;
	}
	else
	{
		LCD_Pin_Data |= LCD_RS;
	}
	Latch_Put(LCD_Pin_Data);
}


void SetLCD_EN(uint8_t stat)
{
	if(stat==0)
	{
		LCD_Pin_Data &= ~LCD_EN;
	}
	else
	{
		LCD_Pin_Data |= LCD_EN;
	}
	Latch_Put(LCD_Pin_Data);
}

void SetLCD_Data(uint8_t Data)
{
	uint8_t lookup[16] =
	{
	   0x0, 0x8, 0x4, 0xC,0x2, 0xA, 0x6, 0xE,
	   0x1, 0x9, 0x5, 0xD,0x3, 0xB, 0x7, 0xF
	};

	Data = (Data&0x0F);

	LCD_Pin_Data &=0xF0;
	LCD_Pin_Data |= lookup[Data];
	Latch_Put(LCD_Pin_Data);
}


void LCD_STROBE(void)
{
	SetLCD_EN(1);
	delay_us(10);
	SetLCD_EN(0);
	delay_us(10);
}

void lcd_write_byte (char Value)
{
	char upperNibble = Value >> 4;
	char lowerNibble = Value & 0x0F;

	SetLCD_Data(upperNibble);
	LCD_STROBE();

	SetLCD_Data(lowerNibble);
	LCD_STROBE();

}

void lcd_command(char cmd)
{
		SetLCD_RS(0);
		delay_us(10);
		lcd_write_byte(cmd);
}

void lcd_data(char ch)
{
	SetLCD_RS(1);//LCD_RS_HI;
	delay_us(10);
	lcd_write_byte(ch);
}


void lcd_init(void)
{
	char cchar[]={0x00,0x10,0x08,0x04,0x02,0x01,0x00,0x00};
	char achar[]={0x1F,0x15,0x15,0x0E,0x04,0x04,0x04,0x00};

	Init_Latch();
	SetLCD_RW(0); /*Always Write To Lcd*/

	delay_ms(15);
	lcd_command(0x30);
	delay_ms(4);
	lcd_command(0x30);
	delay_ms(1);
	lcd_command(0x30);
	delay_ms(1);
	lcd_command(0x02);
	delay_ms(1);


	lcd_command(0x28);delay_ms(1); // configuring LCD as 2 line 5x7 matrix in 4-bit mode
	lcd_command(0x0C);delay_ms(1);
	lcd_command(0x08);delay_ms(1);
	lcd_command(0x01);delay_ms(1);
	lcd_command(0x02);delay_ms(1);
	//lcd_command(0x06);
	lcd_command(0x0c);delay_ms(1); // Display on, Cursor blinking

	lcd_LoadCustomChar(cchar,1);
	lcd_LoadCustomChar(achar,2);
}

void lcd_gotoxy(char x, char y)
{
	register char DDRAMAddr;

	// remap lines into proper order
	switch(y)
	{
	case 0: DDRAMAddr = LCD_LINE0_DDRAMADDR+x; break;
	case 1: DDRAMAddr = LCD_LINE1_DDRAMADDR+x; break;
	case 2: DDRAMAddr = LCD_LINE2_DDRAMADDR+x; break;
	case 3: DDRAMAddr = LCD_LINE3_DDRAMADDR+x; break;
	default: DDRAMAddr = LCD_LINE0_DDRAMADDR+x;
	}

	// set data address
	lcd_command(1<<LCD_DDRAM | DDRAMAddr);
}

void lcd_Home(void)
{
	// move cursor to home
	lcd_command(1<<LCD_HOME);
}

void lcd_cls(void)
{
	// clear LCD
	lcd_command(1<<LCD_CLR);
}

void lcd_LoadCustomChar(char* lcdCustomCharArray, char lcdCharNum)
{
	char i;

	// multiply the character index by 8
	lcdCharNum = (lcdCharNum<<3);	// each character occupies 8 bytes

	// copy the 8 bytes into CG (character generator) RAM
	for(i=0; i<8; i++)
	{
		// set CG RAM address
		lcd_command((1<<LCD_CGRAM) | (lcdCharNum+i));
		// write character data
		lcd_data(*(lcdCustomCharArray+i));
	}
	lcd_gotoxy(0,0);
}

void lcd_putc(char c)
{
   lcd_data(c);
}

void lcd_puts(char* data)
{
	while(*data!=0){
		lcd_data(*data++);
		}
}

